package com.persagy.dptool;

import com.rabbitmq.client.Channel;
import com.rabbitmq.client.Connection;
import com.rabbitmq.client.ConnectionFactory;
import javafx.application.Platform;
import javafx.concurrent.Task;

import javax.jms.TopicSession;
import java.io.File;
import java.util.Map;

public class TaskFactory {
    public static String[] prefixStr = {"- Error", "- Warn"};

    public static Task<Boolean> checkConfig(PrimaryController controller, File configFile, File propertyFile) {
        return new Task<Boolean>() {
            @Override
            protected Boolean call() throws Exception {
                Platform.runLater(()->{
                    controller.txaContent.setText("");
                    controller.setDisable(true, 1);
                    controller.piState.setVisible(true);
                    controller.lblState.setText("准备校验...");
                });

                if(configFile != null && configFile.isFile()) {
                    Platform.runLater(()->{
                        controller.lblState.setText("校验config.properties文件...");
                    });
                    Thread.sleep(500);

                    Map<String, String> configMap = CommonUtil.property2Map(configFile);
                    String propertyFileCheckResult = checkConfigFile(controller, configMap);
                    Platform.runLater(()->{
                        controller.txaContent.setText(propertyFileCheckResult);
                    });
                }else {
                    Platform.runLater(()->{
                        controller.lblState.setText("未找到config.properties文件，准备校验数据字典文件...");
                    });
                    Thread.sleep(500);
                }

                Platform.runLater(()->{
                    controller.setDisable(false, 1);
                    controller.piState.setVisible(false);
                    controller.lblState.setText("");
                });
                return true;
            }
        };
    }

    /**
     * 校验数据平台properties配置文件
     * @param controller
     * @param configMap
     */
    private static String checkConfigFile(PrimaryController controller, Map<String, String> configMap) {
        StringBuilder sbStr = new StringBuilder("【config.properties配置文件】\n");

        String dictSource = configMap.get("dict.source");
        if(dictSource == null) {
            sbStr.append(getLineString("配置项dict.source缺失；", 0));
        }else if(!"local".equals(dictSource)) {
            sbStr.append(getLineString("配置项dict.source未使用本地数据字典配置；",1));
        }

        String jmsActive = configMap.get("jms.active");
        if("false".equals(jmsActive)) {
            sbStr.append(getLineString("消息中间件配置未打开；", 0));
        }else {
            String jmsChoice = configMap.get("jms.choice");
            if("rabbit".equals(jmsChoice)) {
                // 使用的是rabbitMq中间件
                sbStr.append(checkRabbitMq(configMap));
            }else {
                // 使用是activeMq中间件
                sbStr.append(checkActiveMQ(configMap));
            }
        }

        String imgServiceUrl = configMap.get("image.service.url");
        if(imgServiceUrl == null) {
            sbStr.append(getLineString("配置项image.service.url缺失；", 0));
        }else {
            String imgSystemId = configMap.get("image.service.systemId");
            String imgSecret= configMap.get("image.service.secret");
            if(null != imgSecret && imgSystemId != null) {
                if(!imgSecret.equals(DataDTO.imgKeyMap.get(imgSystemId))) {
                    sbStr.append(getLineString("文件服务systemId与secret不匹配；", 0));
                }else {
                    if(!fileGetTest(imgServiceUrl, imgSystemId)) {
                        sbStr.append(getLineString("文件服务访问不通，请确保生产环境数据平台可访问文件服务；", 0));
                    }
                }
            }else {
                if(null == imgSystemId) {
                    sbStr.append("Error：配置项image.service.systemId缺失；\n");
                }
                if(null == imgSecret) {
                    sbStr.append("Error：配置项image.service.secret缺失；\n");
                }
            }
        }

        String checkResult = sbStr.toString();
        if(checkResult.contains("Error") || checkResult.contains("Warn")) {
            checkResult = checkResult + "\n";
        }else {
            checkResult = checkResult + "未发现异常配置。\n\n";
        }

        return checkResult;
    }

    private static String checkActiveMQ(Map<String, String> configMap) {
        String result = "";

        String brokerurl = configMap.get("jms.brokerurl");
        String userName = configMap.get("jms.userName");
        String password = configMap.get("jms.password");
        String topic = configMap.get("jms.topic");

        if(brokerurl == null || userName == null || password == null || topic == null) {
            result += getLineString("ActiveMq必有配置项值不能为空！请到FTP下载标准数据平台程序，参考其配置文件内容；", 0);
            return result;
        }

        if(!"dataPlatform.broadcast".equals(topic)) {
            result += getLineString("ActiveMq配置项jms.topic值不是默认值dataPlatform.broadcast；", 1);
        }

        try {
            javax.jms.ConnectionFactory factory = new org.apache.activemq.ActiveMQConnectionFactory(userName, password, brokerurl);
            javax.jms.Connection connection = factory.createConnection();
            connection.start();
            javax.jms.Session session = connection.createSession(false, TopicSession.AUTO_ACKNOWLEDGE);
            javax.jms.Topic topicObj = session.createTopic(topic);
            javax.jms.MessageProducer messageProducer = session.createProducer(topicObj);
            messageProducer.setTimeToLive(1000 * 60 * 60 * 2);
        }catch (Exception e) {
            e.printStackTrace();
            result += getLineString("ActiveMq消息中间件服务访问不通！errorMsg=" + e.getMessage() + " 请确保生产环境消息中间件服务能够正常访问。", 0);
        }

        return result;
    }

    /**
     * 检查rabbitMQ服务
     * @param configMap
     * @return
     */
    private static String checkRabbitMq(Map<String, String> configMap) {
        String result = "";
        String host = configMap.get("rabbit.host");
        String port = configMap.get("rabbit.port");
        String virtualhost = configMap.get("rabbit.virtualhost");
        String userName = configMap.get("rabbit.userName");
        String password = configMap.get("rabbit.password");
        String topic = configMap.get("rabbit.topic");
        String routingKey = configMap.get("rabbit.routingKey");

        if(host == null || port == null || virtualhost == null || userName == null || password == null || topic == null || routingKey == null) {
            result += getLineString("RabbitMq必有配置项值不能为空！请到FTP下载标准数据平台程序，参考其配置文件内容；", 0);
            return result;
        }

        if(!"dataPlatform.broadcast".equals(topic)) {
            result += getLineString("RabbitMq配置项rabbit.topic值不是默认值dataPlatform.broadcast；", 1);
        }

        if(!"dataPlatform".equals(routingKey)) {
            result += getLineString("RabbitMq配置项rabbit.routingKey值不是默认值dataPlatform；", 1);
        }

        try {
            ConnectionFactory factory = new ConnectionFactory();
            factory.setUsername(userName);
            factory.setPassword(password);
            factory.setHost(host);
            factory.setPort(Integer.parseInt(port));
            factory.setVirtualHost(virtualhost);

            Connection connection = factory.newConnection();
            Channel channel = connection.createChannel();
            channel.exchangeDeclare(topic, "topic", true);

//            channel.basicPublish(topic, routingKey, null, "data-platform-tool-testMsg".getBytes(Charset.forName("UTF-8")));

        }catch (Exception e) {
            e.printStackTrace();
            result += getLineString("RabbitMq消息中间件服务访问不通！errorMsg=" + e.getMessage() + " 请确保生产环境消息中间件服务能够正常访问。", 0);
        }

        return result;
    }

    private static String getLineString(String str, int prefixIndex) {
        return  prefixStr[prefixIndex] + "：" + str + "\n";
    }

    /**
     * 测试文件服务file_get接口
     * @param base
     * @param sysId
     * @return
     */
    private static boolean fileGetTest(String base, String sysId) {
        boolean result = false;
        String url = base + File.separator + "common/file_get?key=dPfToOltEsTkey&systemId=" + sysId;
        try {
            byte[] byteData = CommonUtil.httpGetFile(url);
            if(byteData != null && ifNotExists(byteData)) {
                result = true;
            }
        }catch (Exception e){}

        return result;
    }

    /**
     * 判断byte数组是否包含“not existed”字符串
     * @param byteData
     * @return true-包含；false-不包含
     */
    private static boolean ifNotExists(byte[] byteData) {
        String s = new String(byteData);
        return s.contains("not existed");
    }
}
